@@ -101,3 +101,12 @@ def is_pytest() -> bool:
             return True
 
     return False  # pragma: no cover
+
+
+def is_ci() -> bool:
+    value = os.environ.get('CI', None)
+
+    if value is None:
+        return False
+
+    return value in {'1', 'true'}
