@@ -36,5 +36,5 @@ class ProviderStateMap:
         for state in states:
             self.add_state(state)
 
-    def as_headers(self, key: str = 'test_client_headers') -> Dict[str, str]:
+    def as_headers(self, key: str = 'pact-provider-state-map') -> Dict[str, str]:
         return {key: ProviderStates(__root__=self.states).json()}
