@@ -21,23 +21,15 @@ with transaction(auto_apply=False) as ops:
 
 """
 
-import itertools
+from __future__ import annotations
+
+import abc
 from contextlib import contextmanager
 from contextvars import ContextVar
 from enum import Enum
-from typing import Callable, Generic, Optional, TypeVar
-
-try:
-    from transaction import Transaction, TransactionManager  # noqa: WPS433 - nested import
-    from transaction.interfaces import IDataManager  # noqa: WPS433 - nested import
-    from transaction._manager import _new_transaction  # noqa: WPS436, WPS450, WPS433 - nested import
-    from zope.interface import implementer  # noqa: WPS433 - nested import
-except ImportError as exc:
-    raise Exception('You need to install the `transaction` extra to use the transaction features') from exc
-
+from typing import Callable, Generic, List, Optional, TypeVar
 
 txns = ContextVar('txns', default=[])
-id_counter = itertools.count()
 
 
 def reset_context():  # pragma: no cover
@@ -45,56 +37,109 @@ def reset_context():  # pragma: no cover
     txns.set([])
 
 
-class TransactionExt(Transaction):
-    def _free(self):
-        # We don't want to free any resources after a nested transaction is commited or aborted.
-        # Otherwise the parent transaction won't be able to abort it.
-        pass
-
-
-class HierarchicalTransactionManager(TransactionManager):
+class TransactionManager:
     @property
-    def transactions(self):
+    def transactions(self) -> List[Transaction]:
         # We store transactions of a hierarchie in a ContextVar to make sure
         # there are no conflicts in an async context.
         return txns.get()
 
-    def __exit__(self, t, v, tb):
-        super().__exit__(t, v, tb)
-        self.transactions.pop()
-
-    def begin(self):
-        # We overload this method to remove a check and join child transactions
-        # to the parent transaction as a resource.
-        parent_transaction = self._txn
-        txn = self._txn = TransactionExt(self._synchs, self)  # noqa: WPS429
-        if parent_transaction is not None:
-            # We wrap the transaction to give it the same interface as a resource, and link it
-            # to the parent transaction. That way, the parent transaction will be able to abort its
-            # child transactions if any of them fails.
-            parent_transaction.join(TransactionResource(txn))
-        _new_transaction(txn, self._synchs)
-        return txn
-
     @property
-    def _txn(self):
+    def current_txn(self) -> Optional[Transaction]:
         if self.transactions:
             # Return the current transaction.
             return self.transactions[-1]
         return None
 
-    @_txn.setter
-    def _txn(self, value):
-        if value is not None:
-            self.transactions.append(value)
+    @current_txn.setter
+    def current_txn(self, value: Transaction):
+        self.transactions.append(value)
+
+    def __enter__(self) -> Transaction:
+        parent_transaction = self.current_txn
+        self.current_txn = Transaction()
+        if parent_transaction is not None:
+            # We wrap the transaction to give it the same interface as a resource, and link it
+            # to the parent transaction. That way, the parent transaction will be able to abort its
+            # child transactions if any of them fails.
+            parent_transaction.join(TransactionResourceManager(self.current_txn))
+        return self.current_txn
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if exc_value is None:
+            self.current_txn.commit()
+        else:
+            self.current_txn.abort()
+        self.transactions.pop()
+
+
+class Status(Enum):
+    active = 'Active'
+    committed = 'Committed'
+    commitfailed = 'Commit failed'
+
+
+class Transaction:
+    _resources: List[ResourceManager]
+
+    def __init__(self):
+        self.status = Status.active
+        self._resources = []
+
+    def join(self, resource: ResourceManager):
+        assert self.status is Status.active  # noqa: S101
+        self._resources.append(resource)
+
+    def commit(self):
+        assert self.status is Status.active  # noqa: S101
+
+        try:
+            self._commit_resources()
+            self.status = Status.committed
+        except Exception as e:  # pragma: no cover
+            self.status = Status.commitfailed
+            raise e
+
+    def abort(self):
+        exceptions = []
+        for rm in reversed(self._resources):
+            try:
+                rm.abort()
+            except Exception as e:  # pragma: no cover
+                exceptions.append(e)
+        if exceptions:  # pragma: no cover
+            raise Exception(exceptions)
+
+    def _commit_resources(self):
+        try:
+            for rm in self._resources:
+                rm.commit()
+        except Exception:  # pragma: no cover
+            # If an error occurs committing a transaction, we try
+            # to revert the changes in each of the resource managers.
+            for rm in reversed(self._resources):  # noqa: WPS440
+                rm.abort()  # noqa: WPS441
+
+
+class ResourceManager(abc.ABC):
+    def __init__(self, txn: Transaction):
+        self.txn = txn
+
+    @abc.abstractmethod
+    def abort(self):  # pragma: no cover
+        ...
+
+    @abc.abstractmethod
+    def commit(self):  # pragma: no cover
+        ...
 
 
 @contextmanager
 def transaction(auto_apply: bool = True):
-    manager = HierarchicalTransactionManager(explicit=True)
+    manager = TransactionManager()
 
     with manager as txn:
-        ops = OperationManager()
+        ops = OperationManager(txn)
         txn.join(ops)
 
         yield ops
@@ -156,11 +201,12 @@ class Operation(Generic[T]):
         self.state = OperationState.reset
 
 
-@implementer(IDataManager)
-class OperationManager:  # noqa: WPS214 - too many methods
-    def __init__(self):
+class OperationManager(ResourceManager):  # noqa: WPS214 - too many methods
+    operations: List[Operation]
+
+    def __init__(self, txn: Transaction):
+        super().__init__(txn)
         self.operations = []
-        self._id = next(id_counter)
 
     def add(self, operation: Operation):
         self.operations.append(operation)
@@ -169,7 +215,7 @@ class OperationManager:  # noqa: WPS214 - too many methods
         for op in self.operations:
             op.apply()
 
-    def abort(self, txn: Transaction):
+    def abort(self):
         for op in reversed(self.operations):
             if op.state == OperationState.applied:
                 op.rollback()
@@ -179,45 +225,13 @@ class OperationManager:  # noqa: WPS214 - too many methods
             if op.state != OperationState.applied:
                 raise OperationNotApplied(op.state)
 
-    def sortKey(self):  # noqa: N802
-        return self._id
-
-    # Methods under this comment are required as the `IDataManager` interface doesn't define any behavior
-    # https://zope.readthedocs.io/en/latest/zdgbook/ComponentsAndInterfaces.html#zope-components
-
-    def tpc_begin(self, txn):
-        pass
-
-    def tpc_vote(self, txn):
-        pass
-
-    def tpc_finish(self, txn):
-        pass
-
-    def commit(self, txn):
+    def commit(self):
         pass
 
 
-@implementer(IDataManager)
-class TransactionResource:
-    def __init__(self, txn: Transaction):
-        self.txn = txn
-        self._id = next(id_counter)
-
-    def abort(self, txn: Transaction):
+class TransactionResourceManager(ResourceManager):
+    def abort(self):
         self.txn.abort()
 
-    def sortKey(self):  # noqa: N802
-        return self._id
-
-    def commit(self, txn: Transaction):
-        pass
-
-    def tpc_begin(self, txn):
-        pass
-
-    def tpc_vote(self, txn):
-        pass
-
-    def tpc_finish(self, txn):
+    def commit(self):
         pass
